package com.sun.org.glassfish.external.statistics.impl;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Map;
import java.lang.reflect.Proxy;
import java.lang.reflect.InvocationHandler;
import com.sun.org.glassfish.external.statistics.BoundaryStatistic;

public final class BoundaryStatisticImpl extends StatisticImpl implements BoundaryStatistic, InvocationHandler
{
    private final long lowerBound;
    private final long upperBound;
    private final BoundaryStatistic bs;
    
    public BoundaryStatisticImpl(long lower, long upper, String name, String unit, String desc, long startTime, long sampleTime) {
        super(name, unit, desc, startTime, sampleTime);
        this.bs = (BoundaryStatistic)Proxy.newProxyInstance(BoundaryStatistic.class.getClassLoader(), new Class[] { BoundaryStatistic.class }, this);
        this.upperBound = upper;
        this.lowerBound = lower;
    }
    
    public synchronized BoundaryStatistic getStatistic() {
        return this.bs;
    }
    
    @Override
    public synchronized Map getStaticAsMap() {
        Map<String, Long> m = super.getStaticAsMap();
        m.put("lowerbound", Long.valueOf(this.getLowerBound()));
        m.put("upperbound", Long.valueOf(this.getUpperBound()));
        return m;
    }
    
    public synchronized long getLowerBound() {
        return this.lowerBound;
    }
    
    public synchronized long getUpperBound() {
        return this.upperBound;
    }
    
    @Override
    public synchronized void reset() {
        super.reset();
        this.sampleTime = -1L;
    }
    
    public Object invoke(Object proxy, Method m, Object[] args) throws Throwable {
        this.checkMethod(m);
        Object result;
        try {
            result = m.invoke(this, args);
        }
        catch (InvocationTargetException e) {
            throw e.getTargetException();
        }
        catch (Exception e2) {
            throw new RuntimeException("unexpected invocation exception: " + e2.getMessage());
        }
        return result;
    }
}
